CONST FALSE = 0
CONST TRUE = NOT FALSE

DECLARE SUB ParseCommandLine (ComPort%, BaudRate%, TargetPosition%, ChanNum%)
DECLARE SUB StartAutoBaudSequence (ComPort%, BaudRate%)
DECLARE SUB DelayInSecond (Delay!)
DECLARE SUB SendString (OBUF$)
DECLARE SUB SendCharacter (OBUF$)
DECLARE SUB ReadCharacter (IBUF$)
DECLARE SUB ReadParameters (InputArray() AS STRING, count%)
DECLARE SUB DisplayHelpScreen ()
DECLARE SUB MainProgram (TargetPosition%, ChanNum%)

ON ERROR GOTO Handler

CALL ParseCommandLine(CP%, BD%, TP%, CHNB%)     'CP% is the com port number.
                                                'BD% is the baud rate.
                                                'TP% is the FW target position.
                                                'CHNB% is the mono channel number.

CALL StartAutoBaudSequence(CP%, BD%)

CALL MainProgram(TP%, CHNB%)

END

Handler:
SELECT CASE ERR
   CASE 53
      PRINT "Can't Open FILTER.INI file."
      PRINT "Please run FW_SETUP.BAT first..."
      PRINT
      GOTO Quit
   CASE ELSE
      PRINT "Error "; ERR; " has occured, terminating program"
      GOTO Quit
END SELECT
  

Quit:
STOP

SUB DelayInSecond (Delay)
     T0 = TIMER
     WHILE ((TIMER - T0) < Delay)
     WEND
END SUB

SUB DisplayHelpScreen
     PRINT
     PRINT "Correct Syntax:"
     PRINT
     PRINT "   FILTER /n"
     PRINT
     PRINT "        n  -> target position ('/' is optional)"
     PRINT

     STOP
END SUB

SUB MainProgram (TargetPosition%, ChanNum% )

     DIM InputArray(1 TO 3)  AS STRING

     PRINT
     PRINT "-------------------------------------------------------"
     PRINT


REM ----------------------------------------------------------------------
REM     Get and Display Current FW Position
REM ----------------------------------------------------------------------

REM     OBUF$ = "Z4,0"      'Z4 is the command to read current filter position.
                         '0 is the monochromator number. In this case it
                         'shoud be always 0.

     OBUF$ = "Z4," + STR$(ChanNum%)
     CALL SendString(OBUF$)        'Send the output string to the filter wheel

     CALL ReadCharacter(ACK$)      'Wait for the confirmation from the FW

     IF ACK$ <> "o" THEN
          PRINT "Bad Response, Terminating Program"     'Something about
                                                        'the command is not
                                                        'right.
          END
     ELSE
          CALL ReadParameters(InputArray(), 1)  'Read the rest of the
                                                'response.
     END IF

     PRINT "Filter Wheel Current Position -> "; InputArray(1)

REM ----------------------------------------------------------------------
REM     Move FW to the target position
REM ----------------------------------------------------------------------
REM     OBUF$ = "Z3,0"      'Z3 is the command to move filter.
                         '0 is the monochromator number. In this case it
                         'shoud be always 0.

     OBUF$ = "Z3," + STR$(ChanNum%)
     OBUF$ = OBUF$ + "," + STR$(TargetPosition%)        'The final parameter
                                                        'for this command
                                                        'is the new position.

     CALL SendString(OBUF$)        'Send the output string to the filter wheel

     CALL ReadCharacter(ACK$)      'Wait for the confirmation from the FW

     IF ACK$ <> "o" THEN
          PRINT "Bad Response, Terminating Program"     'Something about
                                                        'the command is not
                                                        'right.
          END
     END IF

REM ----------------------------------------------------------------------
REM     Test if FW is still moving
REM ----------------------------------------------------------------------

     FWisBusy = TRUE

     DO
        OBUF$ = "Z5"                    'Z5 is the command to check if FW
                                        'is busy or not

        CALL SendString(OBUF$)          'Send the output string to the filter wheel
        CALL ReadCharacter(ACK$)        'Wait for the confirmation from
                                        'the FW

        IF ACK$ <> "o" THEN
                PRINT "Bad Response, Terminating Program"       'Something about
                                                                'the command is not
                                                                'right.
                STOP
        ELSE
                CALL ReadCharacter(ACK$)        'Get the status character
                                                'from the FW
                IF ACK$ = "q" THEN
                        FWisBusy = TRUE
                ELSEIF ACK$ = "z" THEN
                        FWisBusy = FALSE
                ELSE
                        PRINT "Bad Response, received "; ACK$
                        PRINT "Program Terminated"
                        END
                END IF

        END IF
     LOOP UNTIL FWisBusy <> TRUE

REM ----------------------------------------------------------------------
REM     Get and Display Current FW Position
REM ----------------------------------------------------------------------

REM     OBUF$ = "Z4,0"      'Z4 is the command to read current filter position.
                         '0 is the monochromator number. In this case it
                         'shoud be always 0.

     OBUF$ = "Z4," + STR$(ChanNum%)


     CALL SendString(OBUF$)        'Send the output string to the filter wheel

     CALL ReadCharacter(ACK$)      'Wait for the confirmation from the FW

     IF ACK$ <> "o" THEN
          PRINT "Bad Response, Terminating Program"     'Something about
                                                        'the command is not
                                                        'right.
          END
     ELSE
          CALL ReadParameters(InputArray(), 1)  'Read the rest of the
                                                'response.
     END IF

     PRINT "Filter Wheel Final Position   -> "; InputArray(1)


END SUB

SUB ParseCommandLine (ComPort%, BaudRate%, TargetPosition%, ChanNum%)
   CLS                                             'Clear screen

   PRINT "Reading FIL232.INI"

   OPEN "FIL232.INI" FOR INPUT AS #1

   INPUT #1, ComPort%, COMMENT$: PRINT ComPort%, COMMENT$
   INPUT #1, BaudRate%, COMMENT$: PRINT BaudRate%, COMMENT$
   INPUT #1, ChanNum%, COMMENT$: PRINT ChanNum%, COMMENT$

   CLOSE #1

   InParams$ = COMMAND$
   ParamLen = LEN(InParams$)
    
   InParams$ = LTRIM$(InParams$)                   'Take out any spaces
                                                   'in the beginning.
   ParamLen = LEN(InParams$)
   SlashIndex = INSTR(InParams$, "/")
   Param1$ = RIGHT$(InParams$, ParamLen - SlashIndex)

   FirstLetter$ = LEFT$(Param1$, 1)

   IF ((FirstLetter$ < "0") OR (FirstLetter$ > "6")) THEN
      CALL DisplayHelpScreen
   END IF

   TargetPosition% = VAL(FirstLetter$)
END SUB

SUB ReadCharacter (IBUF$)
      IBUF$ = INPUT$(1, #2)
END SUB

SUB ReadParameters (Params() AS STRING, count%)
     FOR I = 1 TO count%
          INPUT #2, Params(I)
     NEXT
END SUB

SUB SendCharacter (OBUF$)
     PRINT #2, OBUF$;
END SUB

SUB SendString (OBUF$)
     PRINT #2, OBUF$
END SUB

SUB StartAutoBaudSequence (ComPort%, BaudRate%)
     OKToQuit = FALSE
     PRINT "<Press any key to stop autobaud>"
DO
     CPort$ = LTRIM$(STR$(ComPort%))
     BRate$ = LTRIM$(STR$(BaudRate%))
     ComParams$ = "com" + CPort$ + ":" + BRate$ + ",N,8,1,RS,DS0"
     OPEN ComParams$ FOR RANDOM AS #2
    
     OKToExit = FALSE
     DO

        CALL DelayInSecond(.5)           'Wait a little'
        WHILE (LOC(2) > 0)               'Flush Input Buffer'
          JUNK$ = INPUT$(1, #2)
        WEND

        PRINT #2, CHR$(248);             'Send Hi IQ Mode Pseudo Commnad'
        PRINT #2, CHR$(222);             'Send Software Reboot if Hung'
        CALL DelayInSecond(.5)           'Wait a little'

        WHILE (LOC(2) > 0)               'Flush input'
          JUNK$ = INPUT$(1, #2): SOUND 2000, 5
        WEND

          REM
          REM     Begin Auto Baud Sequence
          REM

        DO
          PRINT #2, " "; : PRINT ".";
          SOUND 400, 1
          CALL DelayInSecond(.5)                            'Wait a little'
          IF INKEY$ <> "" THEN Interrupt = TRUE             'Check if the user wants to stop
        LOOP WHILE (LOC(2) <= 0) AND Interrupt = FALSE      'No Response. Try Again'

        IF (Interrupt = FALSE) THEN               'Received a character
          ICH$ = INPUT$(1, #2): SOUND 1000, 2
          PRINT : PRINT "Received ", ICH$
        ELSE                                      'User interrupt
          PRINT
          INPUT "Do you want to quit the program (Y or N)? ", Response$
        END IF

        IF (Response$ = "Y") OR (Response$ = "y") THEN    'Request to quit
          CLS
          END
        ELSEIF Interrupt = TRUE THEN              'Request to start over
          ICH$ = ""                               'Skip case select
          OKToExit = TRUE                         'Break this loop
          CLOSE #2
        END IF
        Interrupt = FALSE                         'Reset Interrupt

        SELECT CASE ICH$
          CASE "*"
               CALL DelayInSecond(.5)          'Wait a little'
               INPUT #2, BOOTMSG$              'Flush out Boot Message'
               PRINT BOOTMSG$
               PRINT #2, CHR$(247);            'Send Hi IQ ID'
               ICH$ = INPUT$(1, #2)            'ACK of Hi IQ ID'
               IF (ICH$ <> "=") THEN           'Bad ACK. Try Again'
                    OKToExit = FALSE
               ELSE
                    PRINT "Jump to Main Program"
                    PRINT #2, "O";                  'Send Jump Sequence'
                    PRINT #2, "2000"; : PRINT #2, CHR$(0);
                    WHILE (INPUT$(1, #2) <> "*")    'Wait for Main Program Response'
                    WEND

' Reset the filter params
' Note: This is only neccessary for custom build filter wheel.
'       Under normal condition, HWCFGXXX utility will handle these
'       parameter internally.
'                    CALL SendString("Z7,0,6,960,180,800")
'                    CALL ReadCharacter(ICH$)
'
' Z7    -> Set filter params command
' 0     -> mono system 0
' 6     -> number of filter positions
' 960  -> steps per unit
' 180   -> baclash in steps
' 500   -> steps per second

' Homed in to 0 pos
                    CALL SendCharacter("A")
                    CALL ReadCharacter(ICH$)

                    OKToExit = TRUE
                    OKToQuit = TRUE
               END IF
          CASE "B"
               PRINT "Jump to Main Program"
               PRINT #2, "O";                  'Send Jump Sequence'
               PRINT #2, "2000"; : PRINT #2, CHR$(0);
               WHILE (INPUT$(1, #2) <> "*")    'Wait for Main Program Response'
               WEND
' Reset the filter params...
               CALL SendString("Z7,0,6,2000,240,500")

' Z7    -> Set filter params command
' 0     -> mono system 0
' 6     -> number of filter positions
' 2000  -> steps per unit
' 240   -> baclash in steps
' 500   -> steps per second

               CALL ReadCharacter(ICH$)
' Homed in to 0 pos

               PRINT "Initializing Filter Wheel..."
               PRINT

               CALL SendCharacter("A")
               CALL ReadCharacter(ICH$)

               PRINT "DONE."


               OKToExit = TRUE
               OKToQuit = TRUE
          CASE "F"
               OKToExit = TRUE
               OKToQuit = TRUE
        END SELECT

     LOOP WHILE (OKToExit = FALSE)
LOOP WHILE (OKToQuit = FALSE)

CLS
PRINT "RS-232 Communications Established"
PRINT "Begin Filter Wheel Commands Demo"

END SUB

