'$INCLUDE: 'qbdecl.bas'
DECLARE SUB DisplayError ()
CONST FALSE = 0
CONST TRUE = NOT FALSE

DECLARE SUB ParseCommandLine (Dev%, TargetPosition%)
DECLARE SUB StartAutoBaudSequence (Dev%)
DECLARE SUB DelayInSecond (Delay!)
DECLARE SUB SendString (OBUF$)
DECLARE SUB SendCharacter (OBUF$)
DECLARE SUB ReadCharacter (IBUF$)
DECLARE SUB ReadParameters (InputArray() AS STRING, Count%)
DECLARE SUB DisplayHelpScreen ()
DECLARE SUB MainProgram (TargetPosition%)

COMMON SHARED DevVal%

ON ERROR GOTO Handler

CALL ParseCommandLine(Dev%, TP%)    'Dev% is the GPIB device number.
												'TP% is the FW target position.
CALL StartAutoBaudSequence(Dev%)

CALL MainProgram(TP%)

END

Handler:
SELECT CASE ERR
	CASE 53
		PRINT "Can't Open FILTER.INI file."
		PRINT "Please run FW_SETUP.BAT first..."
		PRINT
		GOTO Quit
	CASE ELSE
		PRINT "Error "; ERR; " has occured, terminating program"
		GOTO Quit
END SELECT
  

Quit:
STOP

END

SUB DelayInSecond (Delay)
	  T0 = TIMER
	  WHILE ((TIMER - T0) < Delay)
	  WEND
END SUB

SUB DisplayError
	  PRINT "ibsta= &H"; HEX$(IBSTA%); " <";
	  IF IBSTA% AND EERR THEN PRINT " ERR";
	  IF IBSTA% AND TIMO THEN PRINT " TIMO";
	  IF IBSTA% AND EEND THEN PRINT " END";
	  IF IBSTA% AND SRQI THEN PRINT " SRQI";
	  IF IBSTA% AND RQS THEN PRINT " RQS";
	  IF IBSTA% AND CMPL THEN PRINT " CMPL";
	  IF IBSTA% AND LOK THEN PRINT " LOK";
	  IF IBSTA% AND RREM THEN PRINT " REM";
	  IF IBSTA% AND CIC THEN PRINT " CIC";
	  IF IBSTA% AND AATN THEN PRINT " ATN";
	  IF IBSTA% AND TACS THEN PRINT " TACS";
	  IF IBSTA% AND LACS THEN PRINT " LACS";
	  IF IBSTA% AND DTAS THEN PRINT " DTAS";
	  IF IBSTA% AND DCAS THEN PRINT " DCAS";
	  PRINT ">"
	  PRINT "iberr= "; IBERR%;
	  IF IBERR% = EDVR THEN PRINT " EDVR <DOS Error>"
	  IF IBERR% = ECIC THEN PRINT " ECIC <Not CIC>"
	  IF IBERR% = ENOL THEN PRINT " ENOL <No Listener>"
	  IF IBERR% = EADR THEN PRINT " EADR <Address error>"
	  IF IBERR% = EARG THEN PRINT " EARG <Invalid argument>"
	  IF IBERR% = ESAC THEN PRINT " ESAC <Not Sys Ctrlr>"
	  IF IBERR% = EABO THEN PRINT " EABO <Op. aborted>"
	  IF IBERR% = ENEB THEN PRINT " ENEB <No GPIB board>"
	  IF IBERR% = EOIP THEN PRINT " EOIP <Async I/O in prg>"
	  IF IBERR% = ECAP THEN PRINT " ECAP <No capability>"
	  IF IBERR% = EFSO THEN PRINT " EFSO <File sys. error>"
	  IF IBERR% = EBUS THEN PRINT " EBUS <Command error>"
	  IF IBERR% = ESTB THEN PRINT " ESTB <Status byte lost>"
	  IF IBERR% = ESRQ THEN PRINT " ESRQ <SRQ stuck on>"
	  IF IBERR% = ETAB THEN PRINT " ETAB <Table Overflow>"
	  PRINT "ibcnt = "; IBCNT%

	  V% = 0: CALL IBONL(BD%, V%)
END SUB

SUB DisplayHelpScreen
	  PRINT
	  PRINT "Correct Syntax:"
	  PRINT
	  PRINT "   FILTER /n"
	  PRINT
	  PRINT "        n  -> target position ('/' is optional)"
	  PRINT

	  STOP
END SUB

SUB MainProgram (TargetPosition%)

	  DIM InputArray(1 TO 3)  AS STRING

	  PRINT
	  PRINT "-------------------------------------------------------"
	  PRINT


REM ----------------------------------------------------------------------
REM     Get and Display Current FW Position
REM ----------------------------------------------------------------------

	  OBUF$ = "Z4,0"      'Z4 is the command to read current filter position.
								 '0 is the monochromator number. In this case it
								 'shoud be always 0.

	  CALL SendString(OBUF$)        'Send the output string to the filter wheel

	  CALL ReadCharacter(ACK$)      'Wait for the confirmation from the FW

	  IF ACK$ <> "o" THEN
			 PRINT "Bad Response, Terminating Program"     'Something about
																		  'the command is not
																		  'right.
			 END
	  ELSE
			 CALL ReadParameters(InputArray(), 1)  'Read the rest of the
																'response.
	  END IF

	  PRINT "Filter Wheel Current Position -> "; InputArray(1)

REM ----------------------------------------------------------------------
REM     Move FW to the target position
REM ----------------------------------------------------------------------
	  OBUF$ = "Z3,0"      'Z3 is the command to move filter.
								 '0 is the monochromator number. In this case it
								 'shoud be always 0.

	  OBUF$ = OBUF$ + "," + STR$(TargetPosition%)        'The final parameter
																		  'for this command
																		  'is the new position.

	  CALL SendString(OBUF$)        'Send the output string to the filter wheel

	  CALL ReadCharacter(ACK$)      'Wait for the confirmation from the FW

	  IF ACK$ <> "o" THEN
			 PRINT "Bad Response, Terminating Program"     'Something about
																		  'the command is not
																		  'right.
			 END
	  END IF

REM ----------------------------------------------------------------------
REM     Test if FW is still moving
REM ----------------------------------------------------------------------

	  FWisBusy = TRUE

	  DO
		  OBUF$ = "Z5"                    'Z5 is the command to check if FW
													 'is busy or not

		  CALL SendString(OBUF$)          'Send the output string to the filter wheel
		  CALL ReadCharacter(ACK$)        'Wait for the confirmation from
													 'the FW

		  IF ACK$ <> "o" THEN
					 PRINT "Bad Response, Terminating Program"       'Something about
																					 'the command is not
																					 'right.
					 STOP
		  ELSE
					 CALL ReadCharacter(ACK$)        'Get the status character
																'from the FW
					 IF ACK$ = "q" THEN
								FWisBusy = TRUE
					 ELSEIF ACK$ = "z" THEN
								FWisBusy = FALSE
					 ELSE
								PRINT "Bad Response, received "; ACK$
								PRINT "Program Terminated"
								END
					 END IF

		  END IF
	  LOOP UNTIL FWisBusy <> TRUE

REM ----------------------------------------------------------------------
REM     Get and Display Current FW Position
REM ----------------------------------------------------------------------

	  OBUF$ = "Z4,0"      'Z4 is the command to read current filter position.
								 '0 is the monochromator number. In this case it
								 'shoud be always 0.

	  CALL SendString(OBUF$)        'Send the output string to the filter wheel

	  CALL ReadCharacter(ACK$)      'Wait for the confirmation from the FW

	  IF ACK$ <> "o" THEN
			 PRINT "Bad Response, Terminating Program"     'Something about
																		  'the command is not
																		  'right.
			 END
	  ELSE
			 CALL ReadParameters(InputArray(), 1)  'Read the rest of the
																'response.
	  END IF

	  PRINT "Filter Wheel Final Position   -> "; InputArray(1)


END SUB

SUB ParseCommandLine (Dev%, TargetPosition%)
	CLS                                             'Clear screen

	PRINT "Reading FILGPIB.INI"

	OPEN "FILGPIB.INI" FOR INPUT AS #1

	INPUT #1, Dev%, COMMENT$: PRINT Dev%, COMMENT$

	CLOSE #1

	InParams$ = COMMAND$
	ParamLen = LEN(InParams$)
	 
	InParams$ = LTRIM$(InParams$)                   'Take out any spaces
																	'in the beginning.
	ParamLen = LEN(InParams$)
	SlashIndex = INSTR(InParams$, "/")
	Param1$ = RIGHT$(InParams$, ParamLen - SlashIndex)

	FirstLetter$ = LEFT$(Param1$, 1)

	IF ((FirstLetter$ < "0") OR (FirstLetter$ > "6")) THEN
		CALL DisplayHelpScreen
	END IF

	TargetPosition% = VAL(FirstLetter$)
END SUB

SUB ReadCharacter (IBUF$)
	SHARED DevVal%

' In case some one did remember to set the correct size
	TCHR$ = SPACE$(1)
	CALL IBRD(DevVal%, TCHR$)
	IBUF$ = TCHR$
	IF (IBSTA% AND EERR) THEN
		PRINT "-IBRD ERROR-"
		CALL DisplayError
		STOP
	END IF
END SUB

SUB ReadParameters (Params() AS STRING, Count%)
	DIM TBUF AS STRING
	SHARED DevVal%
	TBUF = SPACE$(300)

	CALL IBRD(DevVal%, TBUF)
	IF (IBSTA% AND EERR) THEN
		PRINT "-IBRD ERROR-"
		CALL DisplayError
	END IF

	TBUF = LEFT$(TBUF, INSTR(TBUF, CHR$(13)) - 1)
	FOR I = 1 TO Count% STEP 1
		BufLen% = LEN(TBUF)
		ParamEndPos = INSTR(TBUF, ",")
		IF (ParamEndPos <> 0) THEN
			Params(I) = LEFT$(TBUF, ParamEndPos - 1)
			TBUF = RIGHT$(TBUF, (BufLen% - ParamEndPos))
		ELSEIF (BufLen% <> 0) THEN
			Params(I) = TBUF
		ELSE
			Params(I) = ""
		END IF
	NEXT
END SUB

SUB SendCharacter (OBUF$)
	SHARED DevVal%

	CALL IBWRT(DevVal%, OBUF$)
	IF (IBSTA% AND EERR) THEN
		PRINT "-IBWRT ERROR-"
		CALL DisplayError
	END IF
END SUB

SUB SendString (OBUF$)
	SHARED DevVal%

	IBUF$ = OBUF$ + CHR$(13)
	CALL IBWRT(DevVal%, IBUF$)
	IF (IBSTA% AND EERR) THEN
		PRINT "-IBWRT ERROR-"
		CALL DisplayError
	END IF
END SUB

SUB StartAutoBaudSequence (Dev%)
	ICH$ = SPACE$(1)
	DEVICE$ = "DEV" + LTRIM$(STR$(Dev%))
	CALL IBFIND(DEVICE$, DevVal%)
	IF (DevVal% < 0) THEN
		PRINT "-IBFIND ERROR-"
		CALL DisplayError
	END IF

	EOSV% = &HD
	V% = EOSV% + &H1400
	CALL IBEOS(DevVal%, V%)

	' Flush input buffer; we don't really care if it's going to time out or not.
	CALL IBTMO(DevVal%, T300ms)
	CALL IBRD(DevVal%, JUNK$)

	CALL IBTMO(DevVal%, T3s)

	OKToExit = FALSE

	' Send <SP>
	CALL IBWRT(DevVal%, " ")
	IF (IBSTA% AND EERR) THEN
		PRINT "-IBWRT ERROR-"
		CALL DisplayError
	END IF

	DO
	DelayInSecond (.5)
		CALL IBRD(DevVal%, ICH$): SOUND 1000, 2
		IF (IBSTA% AND EERR) THEN
			PRINT "-IBRD ERROR-"
			CALL DisplayError
		END IF

		SELECT CASE ICH$
			CASE "*"
' Reset the filter params...
' Note: This is only neccessary for custom build filter wheel.
'       Under normal condition, HWCFGXXX utility will handle these
'       parameter internally.
'
'                               CALL SendString("Z7,0,6,960,240,500")
'
' Z7    -> Set filter params command
' 0     -> mono system 0
' 6     -> number of filter positions
' 960   -> steps per unit
' 100   -> baclash in steps
' 500   -> steps per second
'
'                                CALL ReadCharacter(ICH$)
' Homed in to 0 pos
				CALL SendCharacter("A")
				CALL IBTMO(DevVal%, TNONE)          'Disable Timeout
				CALL ReadCharacter(ICH$)            ' during init...
				CALL IBTMO(DevVal%, T300ms)         'Restore time out

				OKToExit = TRUE
			CASE "B"
				PRINT "Jump to Main Program"
				OBUF$ = "O2000" + CHR$(0)
				CALL IBWRT(DevVal%, OBUF$)
				IF (IBSTA% AND EERR) THEN
					PRINT "-IBWRT ERROR-"
					CALL DisplayError
				END IF
			CASE "F"
				OKToExit = TRUE
			CASE ELSE
				PRINT "??? Confirm", ICH$
				OKToExit = TRUE
				STOP
		END SELECT

	LOOP WHILE (OKToExit = FALSE)

	CLS
	PRINT "GPIB Communications Established"
	PRINT "Begin Filter Wheel Commands Demo"

END SUB

